-- $Id: SessionService.lua 121855 2011-09-09 19:51:52Z marosas $

local oil = require "oil"
local Openbus = require "openbus.Openbus"

local luuid = require "uuid"

local Session = require "core.services.session.Session"
local SessionPrev = require "core.services.session.Session_Prev"

local Log = require "openbus.util.Log"
local Utils = require "openbus.util.Utils"

local oop = require "loop.base"

local ComponentContext = require "scs.core.ComponentContext"
local AdaptiveReceptacle = require "scs.adaptation.AdaptiveReceptacle"

local tostring = tostring
local table    = table
local pairs    = pairs
local ipairs   = ipairs
local next     = next

local format = string.format

---
--Faceta que disponibiliza a funcionalidade bsica do servio de sesso.
---
module "core.services.session.SessionService"

local acsIDL = Utils.ACCESS_CONTROL_SERVICE_INTERFACE

--------------------------------------------------------------------------------
-- Aliases

local StartupFailedException = "IDL:scs/core/StartupFailed:1.0"


--------------------------------------------------------------------------------
-- Faceta ISessionService
--------------------------------------------------------------------------------

SessionService = oop.class{
  sessions = {},                -- Mapeia o dono da sesso no componente
  observed = {},                -- Mapeia um membro nas sesses que ele faz parte
  invalidMemberIdentifier = "",
}

-- component id
local componentId = {}
componentId.name = "Session"
componentId.major_version = 1
componentId.minor_version = 0
componentId.patch_version = 0
componentId.platform_spec = ""

---
--Cria uma sesso associada a uma credencial. A credencial em questo 
--recuperada da requisio pelo interceptador do servio, e repassada atravs
--do objeto PICurrent.
--
--@param member O membro que est solicitando a criao da sesso e que estar
--inserido na sesso automaticamente.
--
--@return true, a sesso e o identificador de membro da sesso em caso de
--sucesso, ou false, caso contrrio.
---
function SessionService:createSession(member)
  local credential = Openbus:getInterceptedCredential()
  if self.sessions[credential.identifier] then
    Log:warn(format(
        "A credencial {%s, %s, %s} tentou criar uma sesso e esta j existe",
        credential.identifier, credential.owner, credential.delegate))
    return false, nil, self.invalidMemberIdentifier
  end
  -- Cria nova sesso
  local orb = Openbus:getORB()
  local component = ComponentContext(orb, componentId)
  component:addFacet("SessionEventSink_"..Utils.IDL_VERSION,
                      Utils.SESSION_ES_INTERFACE,
                      Session.SessionEventSink())
  component:addFacet("SessionEventSink",
                      Utils.SESSION_ES_INTERFACE_PREV,
                      SessionPrev.SessionEventSink())
  component:addFacet("ISession_"..Utils.IDL_VERSION,
                      Utils.SESSION_INTERFACE,
                      Session.Session())
  component:addFacet("ISession",
                      Utils.SESSION_INTERFACE_PREV,
                      SessionPrev.Session())
  component:updateFacet("IReceptacles",
                      AdaptiveReceptacle.AdaptiveReceptacleFacet())
  component:addReceptacle("AccessControlServiceReceptacle", "IDL:scs/core/IComponent:1.0", true)

  local sessionFacet = component["ISession_"..Utils.IDL_VERSION]
  sessionFacet.identifier = self:generateIdentifier()
  sessionFacet.credential = credential
  sessionFacet.sessionService = self
  component["SessionEventSink_"..Utils.IDL_VERSION].sessionService = self
  self.sessions[credential.identifier] = component
  Log:debug(format("A credencial {%s. %s, %s} criou a sesso %s",
      credential.identifier, credential.owner, credential.delegate,
      sessionFacet.identifier))

  -- A credencial deve ser observada!
  local status, acsFacet = oil.pcall(Utils.getReplicaFacetByReceptacle,
    orb, self.context.IComponent, "AccessControlServiceReceptacle",
    "IAccessControlService_" .. Utils.IDL_VERSION, acsIDL)
  if not status or not acsFacet then
    component:deactivateComponent()
    component = nil
    self.sessions[credentialId] = nil
    return false, nil, self.invalidMemberIdentifier
  end

  if not self.observerId then
    self.observerId = acsFacet:addObserver(self.context.SessionServiceCredentialObserver, {})
  end

  -- Adiciona o membro  sesso
  local memberID = sessionFacet:addMember(member)
  return true, component.IComponent, memberID
end

---
--Notificao de deleo de credencial.
--
--@param credential A credencial removida.
---
function SessionService:credentialWasDeletedById(credentialId)
  -- Remove o membro das sesses que ele participa
  local sessions = self.observed[credentialId]
  if sessions then
    for _, session in pairs(sessions) do
      session:credentialWasDeletedById(credentialId)
    end
    self.observed[credentialId] = nil
  end
  -- Remove a sesso que o membro possui
  local component = self.sessions[credentialId]
  if component then
    Log:debug(format(
        "A credencial %s deixou de ser vlida e sua sesso foi removida",
        credentialId))
    component:deactivateComponent()
    component = nil
    self.sessions[credentialId] = nil
  end
end

---
-- Observa o membro para remov-lo das sesses.
--
-- @param credentialId Identificador da credencial do membro
-- @param session Sesso que o membro no vai mais participar
--
function SessionService:observe(credentialId, session)
  local orb = Openbus:getORB()
  local sessions = self.observed[credentialId]
  if not sessions then
    sessions = {}
    self.observed[credentialId] = sessions
    -- Primeira sesso da credencial, comear a observar
    local status, acsFacet = oil.pcall(Utils.getReplicaFacetByReceptacle,
      orb, self.context.IComponent, "AccessControlServiceReceptacle",
      "IAccessControlService_" .. Utils.IDL_VERSION, acsIDL)
    if status and acsFacet then
      acsFacet:addCredentialToObserver(self.observerId, credentialId)
    else
      if not status then
        Log:error("A credencial no foi adicionada ao observador. Houve um erro ao obter a faceta do Servio de Controle de Acesso: " .. acsFacet)
      else
        Log:error("A credencial no foi adicionada ao observador. O proxy para a faceta do Servio de Controle de Acesso  invlido.")
      end
    end
  end
  sessions[session.identifier] = session
end

---
-- Pra de observar o membro na repectiva sesso.
--
-- Se o membro no estiver mais relacionado com sesses,
-- no observar mais sua credential junto ao ACS.
--
-- @param credentialId Identificador da credencial do membro.
-- @param session Sesso que o membro no vai mais participar.
--
function SessionService:unObserve(credentialId, session)
  local orb = Openbus:getORB()
  local sessions = self.observed[credentialId]
  sessions[session.identifier] = nil
  if not next(sessions) then
    self.observed[credentialId] = nil
    local status, acsFacet = oil.pcall(Utils.getReplicaFacetByReceptacle,
      orb, self.context.IComponent, "AccessControlServiceReceptacle",
      "IAccessControlService_" .. Utils.IDL_VERSION, acsIDL)
    if status and acsFacet then
      acsFacet:removeCredentialFromObserver(self.observerId, credentialId)
    else
      if not status then
        Log:error("A credencial no foi removida do observador. Houve um erro ao obter a faceta do Servio de Controle de Acesso: " .. acsFacet)
      else
        Log:error("A credencial no foi removida do observador. O proxy para a faceta do Servio de Controle de Acesso  invlido.")
      end
    end
  end
end

---
--Gera um identificador de sesso.
--
--@return Um identificador de sesso.
---
function SessionService:generateIdentifier()
  return luuid.new("time")
end

---
--Obtm a sesso associada a uma credencial. A credencial em questo 
--recuperada da requisio pelo interceptador do servio, e repassada atravs
--do objeto PICurrent.
--
--@return A sesso, ou nil, caso no exista sesso para a credencial do membro.
---
function SessionService:getSession()
  local credential = Openbus:getInterceptedCredential()
  local session = self.sessions[credential.identifier]
  if not session then
    Log:warn(format("A credencial {%s, %s, %s} no possui sesso",
        credential.identifier, credential.owner, credential.delegate))
    return nil
  end
  return session.IComponent
end

---
--Procedimento aps a reconexo do servio.
---
function SessionService:expired()

  local acsFacet = Openbus:getAccessControlService()
  if not acsFacet then
    Log:error("No foi possvel reconectar o servio de sesso.O servio de controle de acesso no foi encontrado")
    return false
  end

  -- Registra novamente o observador de credenciais
  self.observerId = acsFacet:addObserver(
      self.context.SessionServiceCredentialObserver, {}
  )
  Log:debug(format(
      "O observador de credenciais foi recadastrado com o identificador %s",
      self.observerId))

  -- Mantm apenas as sesses com credenciais vlidas
  local invalidCredentials = {}
  for credentialId, sessions in pairs(self.observed) do
    if not acsFacet:addCredentialToObserver(self.observerId,
        credentialId) then
      Log:debug(format("A sesso da credencial %s ser removida", credentialId))
      table.insert(invalidCredentials, credentialId)
    else
      Log:debug(format("A sesso da credencial %s ser mantida", credentialId))
    end
  end
  for _, credentialId in ipairs(invalidCredentials) do
    self:credentialWasDeletedById(credentialId)
  end
end

---
--Inicia o componente.
--
--@see scs.core.IComponent#startup
---
function SessionService:startup()
  local acsIComp = Openbus:getACSIComponent()
  local orb = Openbus:Orb()
  local success, conId =
    oil.pcall(self.context.IReceptacles.connect, self.context.IReceptacles,
              "AccessControlServiceReceptacle", acsIComp)
  if not success then
    Log:error(
        "Ocorreu um erro ao conectar com o servio de controle de acesso: ",
        conId)
    error(orb:newexcept{StartupFailedException})
 end
end

--------------------------------------------------------------------------------
-- Faceta ICredentialObserver
--------------------------------------------------------------------------------

Observer = oop.class{}

function Observer:credentialWasDeleted(credential)
  self.context["ISessionService_" .. Utils.IDL_VERSION]:credentialWasDeletedById(credential.identifier)
end
